{----------------------------------------------------------------------------
|
| Library: Envision
|
| Module: EnTwain
|
| Description: Twain types and routines.
|              See Twain developer's toolkit for documentation of types.
|              www.twain.org
|
| History: Feb 28, 1999. Michel Brazeau, first version
|
|---------------------------------------------------------------------------}
unit EnTwain;

{$I Envision.Inc}

interface

uses
    Windows,  { for THandle }
    SysUtils, { for Exception }
    EnMisc;   { for TImageFormat }

const

TWRC_SUCCESS           = 0;
TWRC_FAILURE           = 1;
TWRC_CANCEL            = 3;
TWRC_CHECKSTATUS       = 2;
TWRC_DSEVENT           = 4;
TWRC_NOTDSEVENT        = 5;
TWRC_XFERDONE          = 6;


TWCC_SUCCESS           = 0;

DG_CONTROL             = $00001;
DG_IMAGE               = $00002;

DAT_CAPABILITY         = $00001;
DAT_EVENT              = $00002;
DAT_PENDINGXFERS       = $00005;
DAT_USERINTERFACE      = $00009;
DAT_IMAGEINFO          = $00101;
DAT_IMAGELAYOUT        = $00102;
DAT_IMAGENATIVEXFER    = $00104;


TWON_PROTOCOLMINOR     = 5;
TWON_PROTOCOLMAJOR     = 1;

TWON_ONEVALUE          = 5;

TWON_DONTCARE16        = $FFFF;

TWTY_INT16             = $00001;
TWTY_UINT16            = $00004;
TWTY_BOOL              = $00006;
TWTY_FIX32             = $00007;

TWPT_BW                = 0;
TWPT_GRAY              = 1;
TWPT_RGB               = 2;
TWPT_PALETTE           = 3;

TWSX_NATIVE            = 0;

TWLG_DAN               =  0; { Danish                 }
TWLG_DUT               =  1; { Dutch                  }
TWLG_ENG               =  2; { International English  }
TWLG_FCF               =  3; { French Canadian        }
TWLG_FIN               =  4; { Finnish                }
TWLG_FRN               =  5; { French                 }
TWLG_GER               =  6; { German                 }
TWLG_ICE               =  7; { Icelandic              }
TWLG_ITN               =  8; { Italian                }
TWLG_NOR               =  9; { Norwegian              }
TWLG_POR               = 10; { Portuguese             }
TWLG_SPA               = 11; { Spanish                }
TWLG_SWE               = 12; { Swedish                }
TWLG_USA               = 13; { U.S. English           }

TWCY_AFGHANISTAN       = 1001;
TWCY_ALGERIA           =  213;
TWCY_AMERICANSAMOA     =  684;
TWCY_ANDORRA           =  033;
TWCY_ANGOLA            = 1002;
TWCY_ANGUILLA          = 8090;
TWCY_ANTIGUA           = 8091;
TWCY_ARGENTINA         =   54;
TWCY_ARUBA             =  297;
TWCY_ASCENSIONI        =  247;
TWCY_AUSTRALIA         =   61;
TWCY_AUSTRIA           =   43;
TWCY_BAHAMAS           = 8092;
TWCY_BAHRAIN           =  973;
TWCY_BANGLADESH        =  880;
TWCY_BARBADOS          = 8093;
TWCY_BELGIUM           =   32;
TWCY_BELIZE            =  501;
TWCY_BENIN             =  229;
TWCY_BERMUDA           = 8094;
TWCY_BHUTAN            = 1003;
TWCY_BOLIVIA           =  591;
TWCY_BOTSWANA          =  267;
TWCY_BRITAIN           =    6;
TWCY_BRITVIRGINIS      = 8095;
TWCY_BRAZIL            =   55;
TWCY_BRUNEI            =  673;
TWCY_BULGARIA          =  359;
TWCY_BURKINAFASO       = 1004;
TWCY_BURMA             = 1005;
TWCY_BURUNDI           = 1006;
TWCY_CAMAROON          =  237;
TWCY_CANADA            =    2;
TWCY_CAPEVERDEIS       =  238;
TWCY_CAYMANIS          = 8096;
TWCY_CENTRALAFREP      = 1007;
TWCY_CHAD              = 1008;
TWCY_CHILE             =   56;
TWCY_CHINA             =   86;
TWCY_CHRISTMASIS       = 1009;
TWCY_COCOSIS           = 1009;
TWCY_COLOMBIA          =   57;
TWCY_COMOROS           = 1010;
TWCY_CONGO             = 1011;
TWCY_COOKIS            = 1012;
TWCY_COSTA_RICA        =  506;
TWCY_CUBA              =  005;
TWCY_CYPRUS            =  357;
TWCY_CZECHOSLOVAKIA    =   42;
TWCY_DENMARK           =   45;
TWCY_DJIBOUTI          = 1013;
TWCY_DOMINICA          = 8097;
TWCY_DOMINCANREP       = 8098;
TWCY_EASTERIS          = 1014;
TWCY_ECUADOR           =  593;
TWCY_EGYPT             =   20;
TWCY_ELSALVADOR        =  503;
TWCY_EQGUINEA          = 1015;
TWCY_ETHIOPIA          =  251;
TWCY_FALKLANDIS        = 1016;
TWCY_FAEROEIS          =  298;
TWCY_FIJIISLANDS       =  679;
TWCY_FINLAND           =  358;
TWCY_FRANCE            =   33;
TWCY_FRANTILLES        =  596;
TWCY_FRGUIANA          =  594;
TWCY_FRPOLYNEISA       =  689;
TWCY_FUTANAIS          = 1043;
TWCY_GABON             =  241;
TWCY_GAMBIA            =  220;
TWCY_GERMANY           =   49;
TWCY_GHANA             =  233;
TWCY_GIBRALTER         =  350;
TWCY_GREECE            =   30;
TWCY_GREENLAND         =  299;
TWCY_GRENADA           = 8099;
TWCY_GRENEDINES        = 8015;
TWCY_GUADELOUPE        =  590;
TWCY_GUAM              =  671;
TWCY_GUANTANAMOBAY     = 5399;
TWCY_GUATEMALA         =  502;
TWCY_GUINEA            =  224;
TWCY_GUINEABISSAU      = 1017;
TWCY_GUYANA            =  592;
TWCY_HAITI             =  509;
TWCY_HONDURAS          =  504;
TWCY_HONG_KONG         =  852;
TWCY_HUNGARY           =   36;
TWCY_ICELAND           =  354;
TWCY_INDIA             =   91;
TWCY_INDONESIA         =   62;
TWCY_IRAN              =   98;
TWCY_IRAQ              =  964;
TWCY_IRELAND           =  353;
TWCY_ISRAEL            =  972;
TWCY_ITALY             =   39;
TWCY_IVORY_COAST       =  225;
TWCY_JAMAICA           = 8010;
TWCY_JAPAN             =   81;
TWCY_JORDAN            =  962;
TWCY_KENYA             =  254;
TWCY_KIRIBATI          = 1018;
TWCY_KOREA             =   82;
TWCY_KUWAIT            =  965;
TWCY_LAOS              = 1019;
TWCY_LEBANON           = 1020;
TWCY_LIBERIA           =  231;
TWCY_LIBYA             =  218;
TWCY_LIECHTENSTEIN     =   41;
TWCY_LUXENBOURG        =  352;
TWCY_MACAO             =  853;
TWCY_MADAGASCAR        = 1021;
TWCY_MALAWI            =  265;
TWCY_MALAYSIA          =   60;
TWCY_MALDIVES          =  960;
TWCY_MALI              = 1022;
TWCY_MALTA             =  356;
TWCY_MARSHALLIS        =  692;
TWCY_MAURITANIA        = 1023;
TWCY_MAURITIUS         =  230;
TWCY_MEXICO            =    3;
TWCY_MICRONESIA        =  691;
TWCY_MIQUELON          =  508;
TWCY_MONACO            =   33;
TWCY_MONGOLIA          = 1024;
TWCY_MONTSERRAT        = 8011;
TWCY_MOROCCO           =  212;
TWCY_MOZAMBIQUE        = 1025;
TWCY_NAMIBIA           =  264;
TWCY_NAURU             = 1026;
TWCY_NEPAL             =  977;
TWCY_NETHERLANDS       =   31;
TWCY_NETHANTILLES      =  599;
TWCY_NEVIS             = 8012;
TWCY_NEWCALEDONIA      =  687;
TWCY_NEWZEALAND        =   64;
TWCY_NICARAGUA         =  505;
TWCY_NIGER             =  227;
TWCY_NIGERIA           =  234;
TWCY_NIUE              = 1027;
TWCY_NORFOLKI          = 1028;
TWCY_NORWAY            =   47;
TWCY_OMAN              =  968;
TWCY_PAKISTAN          =   92;
TWCY_PALAU             = 1029;
TWCY_PANAMA            =  507;
TWCY_PARAGUAY          =  595;
TWCY_PERU              =   51;
TWCY_PHILLIPPINES      =   63;
TWCY_PITCAIRNIS        = 1030;
TWCY_PNEWGUINEA        =  675;
TWCY_POLAND            =   48;
TWCY_PORTUGAL          =  351;
TWCY_QATAR             =  974;
TWCY_REUNIONI          = 1031;
TWCY_ROMANIA           =   40;
TWCY_RWANDA            =  250;
TWCY_SAIPAN            =  670;
TWCY_SANMARINO         =  039;
TWCY_SAOTOME           = 1033;
TWCY_SAUDIARABIA       =  966;
TWCY_SENEGAL           =  221;
TWCY_SEYCHELLESIS      = 1034;
TWCY_SIERRALEONE       = 1035;
TWCY_SINGAPORE         =   65;
TWCY_SOLOMONIS         = 1036;
TWCY_SOMALI            = 1037;
TWCY_SOUTH_AFRICA      =   27;
TWCY_SPAIN             =   34;
TWCY_SRILANKA          =   94;
TWCY_STHELENA          = 1032;
TWCY_STKITTS           = 8013;
TWCY_STLUCIA           = 8014;
TWCY_STPIERRE          =  508;
TWCY_STVINCENT         = 8015;
TWCY_SUDAN             = 1038;
TWCY_SURINAME          =  597;
TWCY_SWAZILAND         =  268;
TWCY_SWEDEN            =   46;
TWCY_SWITZERLAND       =   41;
TWCY_SYRIA             = 1039;
TWCY_TAIWAN            =  886;
TWCY_TANZANIA          =  255;
TWCY_THAILAND          =   66;
TWCY_TOBAGO            = 8016;
TWCY_TOGO              =  228;
TWCY_TONGAIS           =  676;
TWCY_TRINIDAD          = 8016;
TWCY_TUNISIA           =  216;
TWCY_TURKEY            =   90;
TWCY_TURKSCAICOS       = 8017;
TWCY_TUVALU            = 1040;
TWCY_UGANDA            =  256;
TWCY_USSR              =    7;
TWCY_UAEMIRATES        =  971;
TWCY_UNITEDKINGDOM     =   44;
TWCY_USA               =    1;
TWCY_URUGUAY           =  598;
TWCY_VANUATU           = 1041;
TWCY_VATICANCITY       =   39;
TWCY_VENEZUELA         =   58;
TWCY_WAKE              = 1042;
TWCY_WALLISIS          = 1043;
TWCY_WESTERNSAHARA     = 1044;
TWCY_WESTERNSAMOA      = 1045;
TWCY_YEMEN             = 1046;
TWCY_YUGOSLAVIA        =   38;
TWCY_ZAIRE             =  243;
TWCY_ZAMBIA            =  260;
TWCY_ZIMBABWE          =  263;


DAT_IDENTITY           = $00003;
DAT_PARENT             = $00004;
DAT_STATUS             = $00008;

MSG_NULL               = $00000;
MSG_GET                = $00001;
MSG_SET                = $00006;
MSG_RESET              = $00007;
MSG_ENDXFER            = $00701;
MSG_PROCESSEVENT       = $00601;

MSG_XFERREADY          = $00101;
MSG_CLOSEDSREQ         = $00102;
MSG_CLOSEDSOK          = $00103;
{ MB added Nov 21, 2000. Twain 1.8 spec }
MSG_DEVICEEVENT        = $00104; 


MSG_OPENDSM            = $00301;
MSG_CLOSEDSM           = $00302;
MSG_OPENDS             = $00401;
MSG_CLOSEDS            = $00402;
MSG_USERSELECT         = $00403;
MSG_DISABLEDS          = $00501; 
MSG_ENABLEDS           = $00502;

MSG_GETDEFAULT         = $00003;
MSG_GETFIRST           = $00004;
MSG_GETNEXT            = $00005;

{=== commented out by CW
ICAP_PIXELTYPE         = $00101;
ICAP_UNITS             = $00102;
ICAP_XFERMECH          = $00103;
ICAP_BITDEPTH          = $0112b;
ICAP_XRESOLUTION       = $01118;
ICAP_YRESOLUTION       = $01119;
ICAP_AUTOBRIGHT        = $01100; //mdm 02/18/2003
ICAP_BRIGHTNESS        = $01101;
ICAP_CONTRAST          = $01103;
ICAP_THRESHOLD         = $01123;
ICAP_BITDEPTHREDUCTION = $0112C;
ICAP_PHYSICALWIDTH     = $01111; //mdm 02/19/2003
ICAP_PHYSICALHEIGHT    = $01112; //mdm 02/19/2003
}

TWBR_THRESHOLD    = 0;
TWBR_HALFTONE     = 1;
TWBR_CUSTHALFTONE = 2;
TWBR_DIFFUSION    = 3;


TWUN_INCHES            = 0;
TWUN_CENTIMETERS       = 1; //mdm 02/19/2003
TWUN_PICAS             = 2; //mdm 02/19/2003
TWUN_POINTS            = 3; //mdm 02/19/2003
TWUN_TWIPS             = 4; //mdm 02/19/2003
TWUN_PIXELS            = 5; //mdm 02/19/2003

{=== commented out by CW
CAP_XFERCOUNT          = $00001;
CAP_FEEDERENABLED      = $01002;
CAP_FEEDERLOADED       = $01003;
CAP_INDICATORS         = $0100b; //mdm 02/18/2003
}

// == Added by CW
const

{ ICAP_SUPPORTEDSIZES values (SS_ means Supported Sizes) }
  TWSS_NONE     =  0;
  TWSS_A4LETTER =  1;
  TWSS_B5LETTER =  2;
  TWSS_USLETTER =  3;
  TWSS_USLEGAL  =  4;
{ Added 1.5 }
  TWSS_A5      =   5;
  TWSS_B4      =   6;
  TWSS_B6      =   7;
//  TWSS_B     =    8;
// Added 1.7
  TWSS_USLEDGER   =  9;
  TWSS_USEXECUTIVE= 10;
  TWSS_A3         = 11;
  TWSS_B3         = 12;
  TWSS_A6         = 13;
  TWSS_C4         = 14;
  TWSS_C5         = 15;
  TWSS_C6         = 16;
{ Added 1.8 }
  TWSS_4A0        = 17;
  TWSS_2A0        = 18;
  TWSS_A0         = 19;
  TWSS_A1         = 20;
  TWSS_A2         = 21;
  TWSS_A4         = TWSS_A4LETTER;
  TWSS_A7         = 22;
  TWSS_A8         = 23;
  TWSS_A9         = 24;
  TWSS_A10        = 25;
  TWSS_ISOB0      = 26;
  TWSS_ISOB1      = 27;
  TWSS_ISOB2      = 28;
  TWSS_ISOB3      = TWSS_B3;
  TWSS_ISOB4      = TWSS_B4;
  TWSS_ISOB5      = 29;
  TWSS_ISOB6      = TWSS_B6;
  TWSS_ISOB7      = 30;
  TWSS_ISOB8      = 31;
  TWSS_ISOB9      = 32;
  TWSS_ISOB10     = 33;
  TWSS_JISB0      = 34;
  TWSS_JISB1      = 35;
  TWSS_JISB2      = 36;
  TWSS_JISB3      = 37;
  TWSS_JISB4      = 38;
  TWSS_JISB5      = TWSS_B5LETTER;
  TWSS_JISB6      = 39;
  TWSS_JISB7      = 40;
  TWSS_JISB8      = 41;
  TWSS_JISB9      = 42;
  TWSS_JISB10     = 43;
  TWSS_C0         = 44;
  TWSS_C1         = 45;
  TWSS_C2         = 46;
  TWSS_C3         = 47;
  TWSS_C7         = 48;
  TWSS_C8         = 49;
  TWSS_C9         = 50;
  TWSS_C10        = 51;
  TWSS_USSTATEMENT  =  52;
  TWSS_BUSINESSCARD =  53;

{ ICAP_DUPLEX values }
  TWDX_NONE        = 0;
  TWDX_1PASSDUPLEX = 1;
  TWDX_2PASSDUPLEX = 2;

  CAP_CUSTOMBASE     =    $8000; { Base of custom capabilities }

{ all data sources are REQUIRED to support these caps }
  CAP_XFERCOUNT      =    $0001;

{ image data sources are REQUIRED to support these caps }
  ICAP_COMPRESSION   =    $0100;
  ICAP_PIXELTYPE     =    $0101;
  ICAP_UNITS         =    $0102; { default is TWUN_INCHES }
  ICAP_XFERMECH      =    $0103;

{ all data sources MAY support these caps }
  CAP_AUTHOR             =    $1000;
  CAP_CAPTION            =    $1001;
  CAP_FEEDERENABLED      =    $1002;
  CAP_FEEDERLOADED       =    $1003;
  CAP_TIMEDATE           =    $1004;
  CAP_SUPPORTEDCAPS      =    $1005;
  CAP_EXTENDEDCAPS       =    $1006;
  CAP_AUTOFEED           =    $1007;
  CAP_CLEARPAGE          =    $1008;
  CAP_FEEDPAGE           =    $1009;
  CAP_REWINDPAGE         =    $100a;
  CAP_INDICATORS         =    $100b;   { Added 1.1 }
  CAP_SUPPORTEDCAPSEXT   =    $100c;   { Added 1.6 }
  CAP_PAPERDETECTABLE    =    $100d;   { Added 1.6 }
  CAP_UICONTROLLABLE     =    $100e;   { Added 1.6 }
  CAP_DEVICEONLINE       =    $100f;   { Added 1.6 }
  CAP_AUTOSCAN           =    $1010;   { Added 1.6 }
  CAP_THUMBNAILSENABLED  =    $1011;   { Added 1.7 }
  CAP_DUPLEX             =    $1012;   { Added 1.7 }
  CAP_DUPLEXENABLED      =    $1013;   { Added 1.7 }
  CAP_ENABLEDSUIONLY     =    $1014;   { Added 1.7 }
  CAP_CUSTOMDSDATA       =    $1015;   { Added 1.7 }
  CAP_ENDORSER           =    $1016;   { Added 1.7 }
  CAP_JOBCONTROL         =    $1017;   { Added 1.7 }
  CAP_ALARMS             =    $1018;   { Added 1.8 }
  CAP_ALARMVOLUME        =    $1019;   { Added 1.8 }
  CAP_AUTOMATICCAPTURE   =    $101a;   { Added 1.8 }
  CAP_TIMEBEFOREFIRSTCAPTURE = $101b;   { Added 1.8 }
  CAP_TIMEBETWEENCAPTURES=    $101c;   { Added 1.8 }
  CAP_CLEARBUFFERS       =    $101d;   { Added 1.8 }
  CAP_MAXBATCHBUFFERS    =    $101e;   { Added 1.8 }
  CAP_DEVICETIMEDATE     =    $101f;   { Added 1.8 }
  CAP_POWERSUPPLY        =    $1020;   { Added 1.8 }
  CAP_CAMERAPREVIEWUI    =    $1021;   { Added 1.8 }
  CAP_DEVICEEVENT        =    $1022;   { Added 1.8 }
  CAP_REACQUIREALLOWED   =    $1023;   { Added 1.8 }
  CAP_SERIALNUMBER       =    $1024;   { Added 1.8 }
  CAP_PRINTER            =    $1026;   { Added 1.8 }
  CAP_PRINTERENABLED     =    $1027;   { Added 1.8 }
  CAP_PRINTERINDEX       =    $1028;   { Added 1.8 }
  CAP_PRINTERMODE        =    $1029;   { Added 1.8 }
  CAP_PRINTERSTRING      =    $102a;   { Added 1.8 }
  CAP_PRINTERSUFFIX      =    $102b;   { Added 1.8 }
  CAP_LANGUAGE           =    $102c;   { Added 1.8 }
  CAP_FEEDERALIGNMENT    =    $102d;   { Added 1.8 }
  CAP_FEEDERORDER        =    $102e;   { Added 1.8 }
  CAP_BATTERYMINUTES     =    $1032;   { Added 1.8 }
  CAP_BATTERYPERCENTAGE  =    $1033;   { Added 1.8 }

{ image data sources MAY support these caps }
  ICAP_AUTOBRIGHT              =    $1100;
  ICAP_BRIGHTNESS              =    $1101;
  ICAP_CONTRAST                =    $1103;
  ICAP_CUSTHALFTONE            =    $1104;
  ICAP_EXPOSURETIME            =    $1105;
  ICAP_FILTER                  =    $1106;
  ICAP_FLASHUSED               =    $1107;
  ICAP_GAMMA                   =    $1108;
  ICAP_HALFTONES               =    $1109;
  ICAP_HIGHLIGHT               =    $110a;
  ICAP_IMAGEFILEFORMAT         =    $110c;
  ICAP_LAMPSTATE               =    $110d;
  ICAP_LIGHTSOURCE             =    $110e;
  ICAP_ORIENTATION             =    $1110;
  ICAP_PHYSICALWIDTH           =    $1111;
  ICAP_PHYSICALHEIGHT          =    $1112;
  ICAP_SHADOW                  =    $1113;
  ICAP_FRAMES                  =    $1114;
  ICAP_XNATIVERESOLUTION       =    $1116;
  ICAP_YNATIVERESOLUTION       =    $1117;
  ICAP_XRESOLUTION             =    $1118;
  ICAP_YRESOLUTION             =    $1119;
  ICAP_MAXFRAMES               =    $111a;
  ICAP_TILES                   =    $111b;
  ICAP_BITORDER                =    $111c;
  ICAP_CCITTKFACTOR            =    $111d;
  ICAP_LIGHTPATH               =    $111e;
  ICAP_PIXELFLAVOR             =    $111f;
  ICAP_PLANARCHUNKY            =    $1120;
  ICAP_ROTATION                =    $1121;
  ICAP_SUPPORTEDSIZES          =    $1122;
  ICAP_THRESHOLD               =    $1123;
  ICAP_XSCALING                =    $1124;
  ICAP_YSCALING                =    $1125;
  ICAP_BITORDERCODES           =    $1126;
  ICAP_PIXELFLAVORCODES        =    $1127;
  ICAP_JPEGPIXELTYPE           =    $1128;
  ICAP_TIMEFILL                =    $112a;
  ICAP_BITDEPTH                =    $112b;
  ICAP_BITDEPTHREDUCTION       =    $112c;  { Added 1.5 }
  ICAP_UNDEFINEDIMAGESIZE      =    $112d;  { Added 1.6 }
  ICAP_IMAGEDATASET            =    $112e;  { Added 1.7 }
  ICAP_EXTIMAGEINFO            =    $112f;  { Added 1.7 }
  ICAP_MINIMUMHEIGHT           =    $1130;  { Added 1.7 }
  ICAP_MINIMUMWIDTH            =    $1131;  { Added 1.7 }
  ICAP_FLIPROTATION            =    $1136;  { Added 1.8 }
  ICAP_BARCODEDETECTIONENABLED =    $1137;  { Added 1.8 }
  ICAP_SUPPORTEDBARCODETYPES   =    $1138;  { Added 1.8 }
  ICAP_BARCODEMAXSEARCHPRIORITIES =  $1139;  { Added 1.8 }
  ICAP_BARCODESEARCHPRIORITIES =    $113a;  { Added 1.8 }
  ICAP_BARCODESEARCHMODE       =    $113b;  { Added 1.8 }
  ICAP_BARCODEMAXRETRIES       =    $113c;  { Added 1.8 }
  ICAP_BARCODETIMEOUT          =    $113d;  { Added 1.8 }
  ICAP_ZOOMFACTOR              =    $113e;  { Added 1.8 }
  ICAP_PATCHCODEDETECTIONENABLED =   $113f;  { Added 1.8 }
  ICAP_SUPPORTEDPATCHCODETYPES =    $1140;  { Added 1.8 }
  ICAP_PATCHCODEMAXSEARCHPRIORITIES = $1141;  { Added 1.8 }
  ICAP_PATCHCODESEARCHPRIORITIES    = $1142;  { Added 1.8 }
  ICAP_PATCHCODESEARCHMODE     =    $1143;  { Added 1.8 }
  ICAP_PATCHCODEMAXRETRIES     =    $1144;  { Added 1.8 }
  ICAP_PATCHCODETIMEOUT        =    $1145;  { Added 1.8 }
  ICAP_FLASHUSED2              =    $1146;  { Added 1.8 }
  ICAP_IMAGEFILTER             =    $1147;  { Added 1.8 }
  ICAP_NOISEFILTER             =    $1148;  { Added 1.8 }
  ICAP_OVERSCAN                =    $1149;  { Added 1.8 }
  ICAP_AUTOMATICBORDERDETECTION=    $1150;  { Added 1.8 }
  ICAP_AUTOMATICDESKEW         =    $1151;  { Added 1.8 }
  ICAP_AUTOMATICROTATE         =    $1152;  { Added 1.8 }

{ image data sources MAY support these audio caps }
  ACAP_AUDIOFILEFORMAT  =  $1201;  { Added 1.8 }
  ACAP_XFERMECH         =  $1202;  { Added 1.8 }


const

{$ifdef Win32}
CTwainDLLFileName = 'TWAIN_32.DLL';
{$else}
CTwainDLLFileName = 'TWAIN.DLL';
{$endif}

type

TW_INT8    = ShortInt;
TW_INT16   = smallint;
TW_INT32   = longint;
TW_UINT8   = byte;
TW_UINT16  = word;
TW_UINT32  = longint;
TW_BOOL    = WordBool;

pTW_INT8   = ^TW_INT8;
pTW_INT16  = ^TW_INT16;
pTW_INT32  = ^TW_INT32;
pTW_UINT8  = ^TW_UINT8;
pTW_UINT16 = ^TW_UINT16;
pTW_UINT32 = ^TW_UINT32;
pTW_BOOL   = ^TW_BOOL;

TW_STR32   = array[0..33] of AnsiChar;
pTW_STR32  = ^TW_STR32;

{$Warnings Off}
TW_MEMREF  = Pointer;
{$Warnings On}

TW_HANDLE  = THandle;

{$Warnings Off}
TW_EVENT  = packed record
    pEvent    : TW_MEMREF;
    TWMessage : TW_UINT16;
end;
{$Warnings On}
pTW_EVENT = ^TW_EVENT;

{$Warnings Off}
TW_FIX32 = packed record
    case Word of
    0 : (Long  : Longint);
    1 : (Whole : TW_INT16;
         Frac  : TW_UINT16);
end;
{$Warnings On}
pTW_FIX32  = ^TW_FIX32;

TW_USERINTERFACE = packed record
    ShowUI  : TW_BOOL;
    ModalUI : TW_BOOL;
    hParent : TW_HANDLE;
end;
pTW_USERINTERFACE = ^TW_USERINTERFACE;

TW_IMAGEINFO = packed record
    XResolution     : TW_FIX32;
    YResolution     : TW_FIX32;
    ImageWidth      : TW_INT32;
    ImageLength     : TW_INT32;
    SamplesPerPixel : TW_INT16;
    BitsPerSample   : array[0..7] of TW_INT16;
    BitsPerPixel    : TW_INT16;
    Planar          : TW_BOOL;
    PixelType       : TW_INT16;
    Compression     : TW_UINT16;
end;
pTW_IMAGEINFO = ^TW_IMAGEINFO;

TW_FRAME = packed record
    Left     : TW_FIX32;
    Top      : TW_FIX32;
    Right    : TW_FIX32;
    Bottom   : TW_FIX32;
end;
pTW_FRAME  = ^TW_FRAME;

TW_AFRAME  = array[0..0] of TW_FRAME;
pTW_AFRAME = ^TW_AFRAME;

TW_IMAGELAYOUT = packed record
    Frame          : TW_FRAME;
    DocumentNumber : TW_UINT32;
    PageNumber     : TW_UINT32;
    FrameNumber    : TW_UINT32;
end;

pTW_IMAGELAYOUT = ^TW_IMAGELAYOUT;


TW_PENDINGXFERS = packed record
    Count           : TW_UINT16;
    Reserved        : TW_UINT32;
end;
pTW_PENDINGXFERS = ^TW_PENDINGXFERS;


TW_ONEVALUE = packed record
    ItemType    : TW_UINT16;
    Item        : TW_UINT32;
end;
pTW_ONEVALUE = ^TW_ONEVALUE;

TW_CAPABILITY = packed record
    Cap        : TW_UINT16;
    ConType    : TW_UINT16;
    hContainer : TW_HANDLE;
end;
pTW_CAPABILITY = ^TW_CAPABILITY;

TW_VERSION = packed record
    MajorNum   : TW_UINT16;
    MinorNum   : TW_UINT16;
    Language   : TW_UINT16;
    Country    : TW_UINT16;
    Info       : TW_STR32;
end;
pTW_VERSION = ^TW_VERSION;


TW_IDENTITY = packed record
    Id              : TW_UINT32;
    Version         : TW_VERSION;
    ProtocolMajor   : TW_UINT16;
    ProtocolMinor   : TW_UINT16;
    SupportedGroups : TW_UINT32;
    Manufacturer    : TW_STR32;
    ProductFamily   : TW_STR32;
    ProductName     : TW_STR32;
end;
pTW_IDENTITY = ^TW_IDENTITY;


TW_STATUS = packed record
    ConditionCode : TW_UINT16;
    Reserved      : TW_UINT16;
end;
pTW_STATUS = ^TW_STATUS;


{$Warnings Off}
TDSM_Entry = function( pOrigin : pTW_IDENTITY;
                       pDest   : pTW_IDENTITY;
                       DG      : TW_UINT32;
                       DAT     : TW_UINT16;
                       MSG     : TW_UINT16;
                       pData   : TW_MEMREF) : TW_UINT16; {$ifdef Win32}
                                                          stdcall;
                                                          {$endif}
{$Warnings On}

TTwainData = record

    DllHandle     : THandle;
    DSMEntry      : TDSM_Entry;
    ParentHandle  : THandle;

    AppId         : TW_IDENTITY;
    SourceId      : TW_IDENTITY;

    ShowUI        : Boolean;
    XDpi          : Word;
    YDpi          : Word;
    ImageFormat   : TImageFormat;
end;

ETwainError = class(Exception);

{$Warnings Off}
function CallTriplet( const pSourceId    : PTW_IDENTITY;
                      const pAppId       : PTW_IDENTITY;
                      const DataGroup    : TW_UINT32;
                      const DataType     : TW_UINT16;
                      const Msg          : TW_UINT16;
                      const pData        : TW_MEMREF;
                      const pTwainResult : PTW_UINT16;
                      const DsmEntry     : TDSM_Entry ) : Boolean;
{$Warnings On}

{ set a single value capability. }
procedure SetSingleValueCap( const CapId      : TW_UINT16;
                             const ItemType   : TW_UINT16;
                             const Value      : TW_UINT32;
                             const pSourceId  : pTW_IDENTITY;
                             const pAppId     : pTW_IDENTITY;
                             const DsmEntry   : TDSM_Entry );

{ Abort a TWAIN transfer. Changes from TWAIN state 6 to 5. }
procedure AbortTransfer( const pSourceId  : pTW_IDENTITY;
                         const pAppId     : pTW_IDENTITY;
                         const DsmEntry   : TDSM_Entry );

{ Concludes a transfer with TWAIN. Returns True if there are more images
  to transfer. On return, if PendingTransfers returns False, TWAIN is
  in state 5; if PendingTranfser is True, TWAIN is in State 6.}
procedure ConcludeTransfer( var   PendingTransfers : TW_BOOL;
                            const pSourceId  : pTW_IDENTITY;
                            const pAppId     : pTW_IDENTITY;
                            const DsmEntry   : TDSM_Entry );

procedure ProcessImages( const AcquireEvent : TAcquireEvent;
                         const pSourceId    : pTW_IDENTITY;
                         const pAppId       : pTW_IDENTITY;
                         const DsmEntry     : TDSM_Entry;
                         const CallBackData : LongInt );

procedure GetAllImages( var   Msg          : TMsg;
                        var   TwainQuit    : Boolean;
                        const AcquireEvent : TAcquireEvent;
                        const pSourceId    : pTW_IDENTITY;
                        const pAppId       : pTW_IDENTITY;
                        const DsmEntry     : TDSM_Entry;
                        const CallBackData : LongInt );

{--------------------------------------------------------------------------}

implementation

uses
    {$ifdef __Debug}
    DBugIntf,
    {$endif}
    Messages,  { for WM_SIZE, WM_TIMER }
    EnMsg,    { for msgOutOfResources }
    Classes;  { for EOutOfResources }

procedure RaiseOutOfResources;
begin
    raise EOutOfResources.Create(msgOutOfResources);
end;

{--------------------------------------------------------------------------}
{$Warnings Off}
function CallTriplet( const pSourceId    : PTW_IDENTITY;
                      const pAppId       : PTW_IDENTITY;
                      const DataGroup    : TW_UINT32;
                      const DataType     : TW_UINT16;
                      const Msg          : TW_UINT16;
                      const pData        : TW_MEMREF;
                      const pTwainResult : PTW_UINT16;
                      const DsmEntry     : TDSM_Entry ) : Boolean;
{$Warnings On}
var
    TwainResult : TW_UINT16;
    TwStatus    : TW_STATUS;
begin

    {$Warnings Off}
    TwainResult := DSMEntry(
                           pAppId, pSourceId,
                           DataGroup,
                           DataType,
                           Msg, pData );
    {$Warnings On}

    Result := (TwainResult = TWRC_SUCCESS);

    if pTwainResult <> nil then
        {$WARNINGS OFF}
        pTwainResult^ := TwainResult;
        {$WARNINGS ON}

    if not Result then
    begin
         { for debugging }
         {$Warnings Off}
         DSMEntry( pAppId, pSourceId,
                   DG_CONTROL,
                   DAT_STATUS,
                   MSG_GET,
                   @TwStatus );
         {$Warnings On}

         { in some cases, (ie, when hiding the user interface on
           HP ScanJet 5s), TwainResult = TWRC_CHECKSTATUS, so the result
           seems the be dependent on the status. }
         Result := (TwainResult            = TWRC_CHECKSTATUS) and
                   (TwStatus.ConditionCode = TWCC_SUCCESS);
    end;
end;

{--------------------------------------------------------------------------}

procedure SetSingleValueCap( const CapId      : TW_UINT16;
                             const ItemType   : TW_UINT16;
                             const Value      : TW_UINT32;
                             const pSourceId  : pTW_IDENTITY;
                             const pAppId     : pTW_IDENTITY;
                             const DsmEntry   : TDSM_Entry );
var
    Cap         : TW_CAPABILITY;
    pOneValue   : PTW_ONEVALUE;
begin
    FillChar(Cap, SizeOf(Cap), 0);

    Cap.Cap        := CapId;
    Cap.ConType    := TWON_ONEVALUE;
    Cap.hContainer := GlobalAlloc(GHND, sizeof (TW_ONEVALUE));
    if Cap.hContainer = 0 then
        RaiseOutOfResources;
    try
        pOneValue := GlobalLock(Cap.hContainer);
        try
            {$WARNINGS OFF}
            pOneValue^.ItemType := ItemType;
            pOneValue^.Item     := Value;
            {$WARNINGS ON}
        finally
            GlobalUnlock(Cap.hContainer);
        end;

  {$Warnings Off}
	CallTriplet( pSourceId, pAppId, DG_CONTROL, DAT_CAPABILITY, MSG_SET,
                     @Cap, nil, DsmEntry);
  {$Warnings On}
    finally
	GlobalFree(cap.hContainer);
    end;
end;

{--------------------------------------------------------------------------}

{ Abort a TWAIN transfer. Changes from TWAIN state 6 to 5. }
procedure AbortTransfer( const pSourceId  : pTW_IDENTITY;
                         const pAppId     : pTW_IDENTITY;
                         const DsmEntry   : TDSM_Entry );
var
    PendingXFers     : TW_PENDINGXFERS;
begin
    FillChar(PendingXFers, SizeOf(PendingXFers), 0);
    {$Warnings Off}
    CallTriplet( pSourceId, pAppId,
                 DG_CONTROL, DAT_PENDINGXFERS,
                 MSG_RESET, @PendingXFers, nil, DsmEntry );
    {$Warnings On}
end;

{--------------------------------------------------------------------------}

procedure ConcludeTransfer( var   PendingTransfers : TW_BOOL;
                            const pSourceId  : pTW_IDENTITY;
                            const pAppId     : pTW_IDENTITY;
                            const DsmEntry   : TDSM_Entry );
var
    PendingXFers     : TW_PENDINGXFERS;
    TwainResult      : TW_UINT16;
begin
    { acknowledge end of transfer }
    {$Warnings Off}
    CallTriplet( pSourceId, pAppId,
                 DG_CONTROL, DAT_PENDINGXFERS,
                 MSG_ENDXFER, @PendingXfers, @TwainResult, DsmEntry);
    {$Warnings On}

    if TwainResult = TWRC_SUCCESS then
        { PendingTransfers may be -1, so check <> 0 }
        PendingTransfers := (PendingXfers.Count <> 0)
    else
    begin
        PendingTransfers := False;

        { if Result is False due to an error the transition to state
          5 is not automatic. }
        AbortTransfer(pSourceId, pAppId, DsmEntry);
    end;
end;

{--------------------------------------------------------------------------}

procedure GetAllImages( var   Msg          : TMsg;
                        var   TwainQuit    : Boolean;
                        const AcquireEvent : TAcquireEvent;
                        const pSourceId    : pTW_IDENTITY;
                        const pAppId       : pTW_IDENTITY;
                        const DsmEntry     : TDSM_Entry;
                        const CallBackData : LongInt );
var
    TwainResult      : TW_UINT16;
    hDib             : TW_UINT32;
    PendingTransfers : TW_BOOL;
    ImageInfo        : TW_IMAGEINFO;
begin
    PendingTransfers := True;

    while PendingTransfers do
    begin
        FillChar(ImageInfo, SizeOf(ImageInfo), 0);
        {$Warnings Off}
        CallTriplet( pSourceId, pAppId,
                     DG_IMAGE, DAT_IMAGEINFO,
                     MSG_GET, @ImageInfo, @TwainResult, DsmEntry );
        {$Warnings On}

        if TwainResult = TWRC_SUCCESS then
        begin
            hDib := 0;
            try
                {$Warnings Off}
                CallTriplet( pSourceId, pAppId,
                             DG_IMAGE, DAT_IMAGENATIVEXFER,
                             MSG_GET, @hDib, @TwainResult, DsmEntry );
                {$Warnings On}

                case TwainResult of

                    TWRC_XFERDONE :
                    begin
                        AcquireEvent( hDib,
                                      ImageInfo.XResolution.Whole,
                                      ImageInfo.YResolution.Whole,
                                      CallBackData );

                        ConcludeTransfer( PendingTransfers,
                                          pSourceId, pAppId, DsmEntry );

                        { MB July 31, 1999. Twain message loop not terminating
                          properly }
                        if not PendingTransfers then
                            TwainQuit := True;
                    end;

                    TWRC_CANCEL:
                    begin
                        { Application must still check for pending transfers as
                          specified by TWAIN. }
                        ConcludeTransfer( PendingTransfers,
                                          pSourceId, pAppId, DsmEntry);
                        if PendingTransfers then
                        begin
                            AbortTransfer(pSourceId, pAppId, DsmEntry);
                            PendingTransfers := False;
                        end;

                        TwainQuit        := True;
                    end;

                    else
                    begin
                        { assume any other result is a failure, including
                          TWRC_FAILURE result code.
                          - the transfer failed (e.g. insufficient memory).
                          - hNative is invalid
                          - image data is still pending

                          Still conclude the transfer, but then
                        }

                        ConcludeTransfer( PendingTransfers,
                                          pSourceId, pAppId, DsmEntry);
                        AbortTransfer(pSourceId, pAppId, DsmEntry);
                        PendingTransfers := False;
                        TwainQuit := True;
                    end;
                end; { case }

            finally
                { In some cases under Win16, call GlobalFree raised a
                  fatal GPF, so GlobalSize is used to ensure the hDib
                  is a valid memory handle }

                if (hDib <> 0) and (GlobalSize(hDib) > 0) then
                     GlobalFree(hDib);
            end;
        end;
    end; { while }
end;


{--------------------------------------------------------------------------}

{ processs a message. Returns True if message is processed by twain source }
function TwainProcessMessage( var   Msg          : TMsg;
                              var   TwainQuit    : Boolean;
                              const AcquireEvent : TAcquireEvent;
                              const pSourceId    : pTW_IDENTITY;
                              const pAppId       : pTW_IDENTITY;
                              const DsmEntry     : TDSM_Entry;
                              const CallBackData : LongInt ) : Boolean;
var
    TwainEvent  : TW_EVENT;
    TwainResult : TW_UINT16;
begin
    TwainQuit := False;

    // 15.04.2008 Patrik Brudere: Fix slow Twain-UI-building -------------------
    if (Msg.Message in [WM_SIZE, WM_PAINT]) or (Msg.Message = WM_TIMER) then
    begin
        Result := False;
        Exit;
    end;

    {$Warnings Off}
    TwainEvent.pEvent    := @Msg;
    {$Warnings On}

    TwainEvent.TWMessage := MSG_NULL;

    {$Warnings Off}
    CallTriplet( pSourceId, pAppId,
                 DG_CONTROL, DAT_EVENT,
                 MSG_PROCESSEVENT, @TwainEvent, @TwainResult, DsmEntry );
    {$Warnings On}

    Result := (TwainResult = TWRC_DSEVENT);


    //SendInteger( 'TwainResult', TwainResult);

    //SendInteger('TwainEvent.TWMessage', TwainEvent.TWMessage);

    //if Result then
    begin

        case TwainEvent.TWMessage of

            MSG_XFERREADY :
            begin
                GetAllImages( Msg, TwainQuit, AcquireEvent, pSourceId, pAppId,
                              DsmEntry, CallBackData );
            end;

            MSG_CLOSEDSREQ :
            begin
                TwainQuit := True;
            end;

            MSG_DEVICEEVENT :
            begin
                { Twain 1.8 }
            end;

            MSG_CLOSEDSOK :
            begin
                TwainQuit := True;
            end;

            MSG_NULL :
            begin
                { not message returned from source }
            end;
        end;
    end;

    (*
    { MB Oct 30, 2000. Message loop was not "pumped" when TWRC_NOTDSEVENT
      was received }
    if TwainResult = TWRC_NOTDSEVENT then
    begin
        SendDebug('TwainResult = TWRC_NOTDSEVENT');
        TranslateMessage(Msg);
        DispatchMessage(Msg);
    end;
    *)
end;

{--------------------------------------------------------------------------}

procedure ProcessImages( const AcquireEvent : TAcquireEvent;
                         const pSourceId    : pTW_IDENTITY;
                         const pAppId       : pTW_IDENTITY;
                         const DsmEntry     : TDSM_Entry;
                         const CallBackData : LongInt );
var
    Msg       : TMsg;
    TwainQuit : Boolean;

begin
    TwainQuit := False;

    while (not TwainQuit) and GetMessage(Msg, 0, 0, 0) do
    begin

      	if not TwainProcessMessage( Msg, TwainQuit, AcquireEvent,
                                    pSourceId, pAppId,
                                    DsmEntry, CallBackData) then
        begin
      	    TranslateMessage(Msg);
            DispatchMessage(Msg);
        end;
    end;
end;

{--------------------------------------------------------------------------}

end.
