
/* tga code adapted from Graphic File Formats Encyclopedia book */

#include <io.h>
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "tga.h"

/*
** Define byte counts associated with extension areas for various
** versions of the TGA specification.
*/

#define CBUFSIZE	2048		/* size of copy buffer */
#define RLEBUFSIZ	512		/* size of largest possible RLE packet */


// fp = file pointer, ie, VCL TStream


// MB Added Jul 9, 2001
struct TTgaInfo
{
    SINT32       Width;
    SINT32       Height;
    UINT8        BitsPerPixel;
} TgaInfo;

extern void             GetTgaInfo( FILE *fp, TTgaInfo *TgaInfo )

extern void		ReadTiffStream( FILE *fp );

extern int		CountDiffPixels( char *, int, int );

extern long		CountRLEData( FILE *, unsigned int, unsigned int, int );

extern int		CountSamePixels( char *, int, int );

extern UINT32	        TgaGetPixel( unsigned char *, int );

extern int		OutputTGAFile( FILE *, FILE *, TGAFile * );

extern UINT8	        ReadByte( FILE * );
extern void		ReadCharField( FILE *, char *, int );
extern UINT32	        ReadLong( FILE * );
extern int		ReadRLERow( unsigned char *, int, int, FILE * );
extern UINT16	        ReadShort( FILE * );
extern char		*SkipBlank( char * );


TGAFile			f;				/* control structure of image data */


int		    	inRawPacket;	/* flags processing state for RLE data */
int		    	inRLEPacket;	/* flags processing state for RLE data */
unsigned int	        packetSize;	/* records current RLE packet size in bytes */
char			rleBuf[RLEBUFSIZ];

char		        copyBuf[CBUFSIZE];


void GetTgaInfo( FILE *fp, TTgaInfo *TgaInfo )
{

}


void
ReadTiffStream( FILE *fp )
{
	int	      	fileFound;
	int	      	fileCount;
	int	      	files;
	char		*q;
	FILE		*outFile;
	int	      	i;
	char		fileName[80];
	char		outFileName[80];
	struct stat	statbuf;

	inRawPacket = inRLEPacket = 0;	/* initialize RLE processing flags */
	packetSize = 0;

    /*
    ** It would be nice to be able to read in the entire
    ** structure with one fread, but compiler dependent
    ** structure alignment precludes the simplistic approach.
    ** Instead, fill each field individually, and use routines
    ** that will allow code to execute on various hosts by
    ** recompilation with particular compiler flags.
    **
    ** Start by reading the fields associated with the original
    ** TGA format.
    */
    f.idLength = ReadByte( fp );
    f.mapType = ReadByte( fp );
    f.imageType = ReadByte( fp );
    f.mapOrigin = ReadShort( fp );
    f.mapLength = ReadShort( fp );
    f.mapWidth = ReadByte( fp );
    f.xOrigin = ReadShort( fp );
    f.yOrigin = ReadShort( fp );
    f.imageWidth = ReadShort( fp );
    f.imageHeight = ReadShort( fp );
    f.pixelDepth = ReadByte( fp );
    f.imageDesc = ReadByte( fp );
    memset( f.idString, 0, 256 );
    if ( f.idLength > 0 )
    {
            fread( f.idString, 1, f.idLength, fp );
    }
    /*
    ** Now see if the file is the new (extended) TGA format.
    */
    if ( !fseek( fp, statbuf.st_size - 26, SEEK_SET ) )
    {
            f.extAreaOffset = ReadLong( fp );
            f.devDirOffset = ReadLong( fp );
            fgets( f.signature, 18, fp );
            if ( strcmp( f.signature, "TRUEVISION-XFILE." ) )
            {
                    /*
                    ** Reset offset values since this is not a new TGA file
                    */
                    f.extAreaOffset = 0L;
                    f.devDirOffset = 0L;

                    strcpy( outFileName, fileName );
                    i = strlen( fileName );
                    outFileName[ i - 3 ] = '\0';	/* remove extension */
                    strcat( outFileName, "$$$" );
                    if ( ( outFile = fopen( outFileName, "wb" ) ) != NULL )
                    {
                            if ( OutputTGAFile( fp, outFile, &f ) < 0 )
                            {
                                    fclose( outFile );
                                    unlink( outFileName );
                            }
                            else
                            {
                                    fclose( outFile );
                                    fclose( fp );
                                    fp = (FILE *)0;
                                    unlink( fileName );
                                    rename( outFileName, fileName );
                            }
                    }
                    else
                    {
                            puts( "Unable to create output file." );
                    }
            }
            else puts( "Input file must be original TGA format." );
    }
    else
    {
            puts( "Error seeking to end of file for possible extension data." );
    }
    if ( fp != NULL ) fclose( fp );
}



/*
** Count pixels in buffer until two identical adjacent ones found
*/

int
CountDiffPixels( p, bpp, pixCnt )
char	*p;
int		bpp;
int		pixCnt;
{
	unsigned long	pixel;
	unsigned long	nextPixel;
	int				n;

	n = 0;
	if ( pixCnt == 1 ) return( pixCnt );
	pixel = TgaGetPixel( p, bpp );
	while ( pixCnt > 1 )
	{
		p += bpp;
		nextPixel = TgaGetPixel( p, bpp );
		if ( nextPixel == pixel ) break;
		pixel = nextPixel;
		++n;
		--pixCnt;
	}
	if ( nextPixel == pixel ) return( n );
	return( n + 1 );
}



long
CountRLEData( fp, x, y, bytesPerPixel )
FILE			*fp;
unsigned int	x;
unsigned int	y;
int				bytesPerPixel;
{
	long			n;
	long			pixelCount;
	long			totalPixels;
	unsigned int	value;

	n = 0L;
	pixelCount = 0L;
	totalPixels = (long)x * (long)y;

	while ( pixelCount < totalPixels )
	{
		value = (unsigned int)ReadByte( fp );
		n++;
		if ( value & 0x80 )
		{
			n += bytesPerPixel;
			pixelCount += (value & 0x7f) + 1;
			if ( fread( copyBuf, 1, bytesPerPixel, fp ) != bytesPerPixel )
			{
				puts( "Error counting RLE data." );
				return( 0L );
			}
		}
		else
		{
			value++;
			n += value * bytesPerPixel;
			pixelCount += value;
			if ( fread( copyBuf, bytesPerPixel, value, fp ) != value )
			{
				puts( "Error counting raw data." );
				return( 0L );
			}
		}
	}
	return( n );
}



int
CountSamePixels( p, bpp, pixCnt )
char	*p;
int		bpp;
int		pixCnt;
{
	unsigned long	pixel;
	unsigned long	nextPixel;
	int				n;

	n = 1;
	pixel = TgaGetPixel( p, bpp );
	pixCnt--;
	while ( pixCnt > 0 )
	{
		p += bpp;
		nextPixel = TgaGetPixel( p, bpp );
		if ( nextPixel != pixel ) break;
		++n;
		--pixCnt;
	}
	return( n );
}







/*
** Retrieve a pixel value from a buffer.  The actual size and order
** of the bytes is not important since we are only using the value
** for comparisons with other pixels.
*/

UINT32
TgaGetPixel( p, bpp )
unsigned char	*p;
int				bpp;		/* bytes per pixel */
{
	unsigned long	pixel;

	pixel = (unsigned long)*p++;
	while ( bpp-- > 1 )
	{
		pixel <<= 8;
		pixel |= (unsigned long)*p++;
	}
	return( pixel );
}



int
OutputTGAFile( ifp, ofp, sp )
FILE		*ifp;		/* input file pointer */
FILE		*ofp;		/* output file pointer */
TGAFile		*sp;		/* output TGA structure */
{
	long			byteCount;
	unsigned long	fileOffset;
	int				i;
	int				bytesPerPixel;
	int				bCount;
	int				rleCount;
	unsigned char	*imageBuff;
	unsigned char	*packBuff;
	unsigned char	outType;
	unsigned char	outDepth;
	unsigned char	outDesc;

	/*
	** First, we need to determine what operation is to be performed.
	** We could be run length encoding an uncompressed image, or
	** we could be uncompressing an RLE image.  Or we could be compacting
	** a 32 bit image to a 24 bit image.
	*/
        outType = sp->imageType - 8;
        outDepth = sp->pixelDepth;
        outDesc = sp->imageDesc;

	/*
	** The output file was just opened, so the first data
	** to be written is the standard header based on the
	** original TGA specification.
	*/
	if ( WriteByte( sp->idLength, ofp ) < 0 ) return( -1 );
	if ( WriteByte( sp->mapType, ofp ) < 0 ) return( -1 );
	if ( WriteByte( outType, ofp ) < 0 ) return( -1 );
	if ( WriteShort( sp->mapOrigin, ofp ) < 0 ) return( -1 );
	if ( WriteShort( sp->mapLength, ofp ) < 0 ) return( -1 );
	if ( WriteByte( sp->mapWidth, ofp ) < 0 ) return( -1 );
	if ( WriteShort( sp->xOrigin, ofp ) < 0 ) return( -1 );
	if ( WriteShort( sp->yOrigin, ofp ) < 0 ) return( -1 );
	if ( WriteShort( sp->imageWidth, ofp ) < 0 ) return( -1 );
	if ( WriteShort( sp->imageHeight, ofp ) < 0 ) return( -1 );
	if ( WriteByte( outDepth, ofp ) < 0 ) return( -1 );
	if ( WriteByte( outDesc, ofp ) < 0 ) return( -1 );
	if ( sp->idLength )
	{
		if ( WriteStr( sp->idString, sp->idLength, ofp ) < 0 )
			return( -1 );
	}
	/*
	** Now we need to copy the color map data from the input file
	** to the output file.
	*/
	byteCount = 18 + sp->idLength;
	if ( fseek( ifp, byteCount, SEEK_SET ) != 0 ) return( -1 );
	byteCount = ((sp->mapWidth + 7) >> 3) * (long)sp->mapLength;
	fileOffset = 18 + sp->idLength + byteCount;
	while ( byteCount > 0 )
	{
		if ( byteCount - CBUFSIZE < 0 )
		{
			fread( copyBuf, 1, (int)byteCount, ifp );
			if ( fwrite( copyBuf, 1, (int)byteCount, ofp ) != (int)byteCount )
				return( -1 );
		}
		else
		{
			fread( copyBuf, 1, CBUFSIZE, ifp );
			if ( fwrite( copyBuf, 1, CBUFSIZE, ofp ) != CBUFSIZE )
				return( -1 );
		}
		byteCount -= CBUFSIZE;
	}
	/*
	** Now process the image data.
	*/
	bytesPerPixel = (sp->pixelDepth + 7) >> 3;
	bCount = sp->imageWidth * bytesPerPixel;
	if ( (imageBuff = malloc( bCount )) == NULL )
	{
		puts( "Unable to allocate image buffer" );
		return( -1 );
	}

		/*
		** Uncompress image data
		*/
		for ( i = 0; i < sp->imageHeight; ++i )
		{
			if ( ReadRLERow( imageBuff, bCount, bytesPerPixel, ifp ) < 0 )
			{
				puts("Error reading RLE data." );
				free( imageBuff );
				return( -1 );
			}
			if ( fwrite( imageBuff, 1, bCount, ofp ) != bCount )
			{
				puts("Error writing uncompressed data." );
				free( imageBuff );
				return( -1 );
			}
		}
	free( imageBuff );

	return( 0 );
}





UINT8
ReadByte( fp )
FILE *fp;
{
	UINT8	value;

	fread( &value, 1, 1, fp );
	return( value );
}


void
ReadCharField( fp, p, n )
FILE	*fp;
char	*p;
int		n;
{
	while ( n )
	{
		*p++ = (char)fgetc( fp );	/* no error check, no char conversion */
		--n;
	}
}




UINT32
ReadLong( fp )
FILE *fp;
{
	UINT32	value;

	fread( &value, 4, 1, fp );

	return( value );
}



int
ReadRLERow( p, n, bpp, fp )
unsigned char	*p;
int		n;			/* buffer size in bytes */
int		bpp;		/* bytes per pixel */
FILE	*fp;
{
	unsigned int			value;
	int						i;
	static unsigned char	*q;

	while ( n > 0 )
	{
		if ( inRLEPacket )
		{
			if ( packetSize * bpp > n )
			{
				value = n / bpp;		/* calculate pixel count */
				packetSize -= value;
				n = 0;
			}
			else
			{
				n -= packetSize * bpp;
				value = packetSize;
				packetSize = 0;
				inRLEPacket = 0;
			}
			while ( value > 0 )
			{
				*p++ = rleBuf[0];
				if ( bpp > 1 ) *p++ = rleBuf[1];
				if ( bpp > 2 ) *p++ = rleBuf[2];
				if ( bpp > 3 ) *p++ = rleBuf[3];
				value--;
			}
		}
		else if ( inRawPacket )
		{
			if ( packetSize * bpp > n )
			{
				value = n;
				packetSize -= n / bpp;
				n = 0;
			}
			else
			{
				value = packetSize * bpp;	/* calculate byte count */
				n -= value;
				inRawPacket = 0;
			}
			for ( i = 0; i < value; ++i ) *p++ = *q++;
		}
		else
		{
			/*
			** No accumulated data in buffers, so read from file
			*/
			packetSize = (unsigned int)ReadByte( fp );
			if ( packetSize & 0x80 )
			{
				packetSize &= 0x7f;
				packetSize++;
				if ( packetSize * bpp > n )
				{
					value = n / bpp;		/* calculate pixel count */
					packetSize -= value;
					inRLEPacket = 1;
					n = 0;
				}
				else
				{
					n -= packetSize * bpp;
					value = packetSize;
				}
				if ( fread( rleBuf, 1, bpp, fp ) != bpp ) return( -1 );
				while ( value > 0 )
				{
					*p++ = rleBuf[0];
					if ( bpp > 1 ) *p++ = rleBuf[1];
					if ( bpp > 2 ) *p++ = rleBuf[2];
					if ( bpp > 3 ) *p++ = rleBuf[3];
					value--;
				}
			}
			else
			{
				packetSize++;
				/*
				** Maximum for packetSize is 128 so as long as RLEBUFSIZ
				** is at least 512, and bpp is not greater than 4
				** we can read in the entire raw packet with one operation.
				*/
				if ( fread( rleBuf, bpp, packetSize, fp ) != packetSize )
					return( -1 );
				/*
				** But is there enough room to copy them to our line buffer?
				*/
				if ( packetSize * bpp > n )
				{
					value = n;				/* number of bytes remaining */
					packetSize -= n / bpp;
					inRawPacket = 1;
					n = 0;
				}
				else
				{
					value = packetSize * bpp;	/* calculate byte count */
					n -= value;
				}
				for ( i = 0, q = rleBuf; i < value; ++i ) *p++ = *q++;
			}
		}
	}
	return( 0 );
}




UINT16
ReadShort( fp )
FILE *fp;
{
	UINT16	value;

	fread( &value, 2, 1, fp );

	return( value );
}




char *
SkipBlank( p )
char	*p;
{
	while ( *p != '\0' && (*p == ' ' || *p == '\t') ) ++p;
	return( p );
}



