/*
 * xtif_dir.c
 *
 * Extended TIFF Directory Tag Support.
 *
 *  You may use this file as a template to add your own
 *  extended tags to the library. Only the parts of the code
 *  marked with "XXX" require modification. Three example tags
 *  are shown; you should replace them with your own.
 *
 *  Author: Niles D. Ritter

  History: Jan 09, 2000 MB Modify to allow the value of the tags
                        to be set at run-time


 */

#include "xtiffiop.h"
#include <stdio.h>

/*  Tiff info structure.
 *
 *     Entry format:
 *        { TAGNUMBER, ReadCount, WriteCount, DataType, FIELDNUM,
 *          OkToChange, PassDirCountOnSet, AsciiName }
 *
 *     For ReadCount, WriteCount, -1 = unknown; used for mult-valued
 *         tags and ASCII.
 */

static TIFFFieldInfo xtiffFieldInfo[] = {

    /* XXX Replace these example tags with your own extended tags */

    /* MB the tag numbers should be overriden with SetTiffTagValues */

    { 40000, -1,-1, TIFF_ASCII, FIELD_USER1, TRUE, FALSE, "UserTag1" },
    { 40001, -1,-1, TIFF_ASCII, FIELD_USER2, TRUE, FALSE, "UserTag2" },
    { 40002, -1,-1, TIFF_ASCII, FIELD_USER3, TRUE, FALSE, "UserTag3" },
    { 40003, -1,-1, TIFF_ASCII, FIELD_USER4, TRUE, FALSE, "UserTag4" },
    { 40004, -1,-1, TIFF_ASCII, FIELD_USER5, TRUE, FALSE, "UserTag5" },
    { 40005, -1,-1, TIFF_ASCII, FIELD_USER6, TRUE, FALSE, "UserTag6" },
    { 40006, -1,-1, TIFF_ASCII, FIELD_USER7, TRUE, FALSE, "UserTag7" },
    { 40007, -1,-1, TIFF_ASCII, FIELD_USER8, TRUE, FALSE, "UserTag8" },
    { 40008, -1,-1, TIFF_ASCII, FIELD_USER9, TRUE, FALSE, "UserTag9" },
    { 40009, -1,-1, TIFF_ASCII, FIELD_USER10, TRUE, FALSE, "UserTag10" },
    { 40010, -1,-1, TIFF_ASCII, FIELD_USER11, TRUE, FALSE, "UserTag11" },
    { 40011, -1,-1, TIFF_ASCII, FIELD_USER12, TRUE, FALSE, "UserTag12" },
    { 40012, -1,-1, TIFF_ASCII, FIELD_USER13, TRUE, FALSE, "UserTag13" },
    { 40013, -1,-1, TIFF_ASCII, FIELD_USER14, TRUE, FALSE, "UserTag14" },
    { 40014, -1,-1, TIFF_ASCII, FIELD_USER15, TRUE, FALSE, "UserTag15" },
    { 40015, -1,-1, TIFF_ASCII, FIELD_USER16, TRUE, FALSE, "UserTag16" },
    { 40016, -1,-1, TIFF_ASCII, FIELD_USER17, TRUE, FALSE, "UserTag17" },
    { 40017, -1,-1, TIFF_ASCII, FIELD_USER18, TRUE, FALSE, "UserTag18" },
    { 40018, -1,-1, TIFF_ASCII, FIELD_USER19, TRUE, FALSE, "UserTag19" },
    { 40019, -1,-1, TIFF_ASCII, FIELD_USER20, TRUE, FALSE, "UserTag20" }
};
#define	N(a)	(sizeof (a) / sizeof (a[0]))


static void
_XTIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
{
	xtiff *xt = XTIFFDIR(tif);
	XTIFFDirectory *xd = &xt->xtif_dir;
	int i,num;

	/* call the inherited method */
	if (PARENT(xt,printdir))
		(PARENT(xt,printdir))(tif,fd,flags);

	/* XXX Add field printing here. Replace the three example
         *    tags implemented below with your own.
         */

        /* MB Jan 09, 2000. Don't print anything. This causes
           unresolved reference in Delphi */
	//fprintf(fd,"--User Tags--\n");

	/* Our first example tag may have a lot of values, so we
         * will only print them out if the TIFFPRINT_MYMULTIDOUBLES
         * flag is passed into the print method.
         */
	if (TIFFFieldSet(tif,FIELD_USER1))
	{
		//_TIFFprintAsciiTag(fd,"User Tag 1", xd->UserField1);
	}
}

static int
_XTIFFVSetField(TIFF* tif, ttag_t tag, va_list ap)
{
	xtiff *xt = XTIFFDIR(tif);
	XTIFFDirectory* xd = &xt->xtif_dir;
	int status = 1;
	uint32 v32=0;
	int i=0, v=0;
	va_list ap1 = ap;

	/* va_start is called by the calling routine */

        // MB Jan 09, 2000 Use if statement to avoid having to use
        // constants in a case statement

        if (tag == xtiffFieldInfo[0].field_tag)
            _TIFFsetString(&xd->UserField1, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[1].field_tag)
            _TIFFsetString(&xd->UserField2, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[2].field_tag)
            _TIFFsetString(&xd->UserField3, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[3].field_tag)
            _TIFFsetString(&xd->UserField4, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[4].field_tag)
            _TIFFsetString(&xd->UserField5, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[5].field_tag)
            _TIFFsetString(&xd->UserField6, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[6].field_tag)
            _TIFFsetString(&xd->UserField7, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[7].field_tag)
            _TIFFsetString(&xd->UserField8, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[8].field_tag)
            _TIFFsetString(&xd->UserField9, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[9].field_tag)
            _TIFFsetString(&xd->UserField10, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[10].field_tag)
            _TIFFsetString(&xd->UserField11, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[11].field_tag)
            _TIFFsetString(&xd->UserField12, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[12].field_tag)
            _TIFFsetString(&xd->UserField13, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[13].field_tag)
            _TIFFsetString(&xd->UserField14, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[14].field_tag)
            _TIFFsetString(&xd->UserField15, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[15].field_tag)
            _TIFFsetString(&xd->UserField16, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[16].field_tag)
            _TIFFsetString(&xd->UserField17, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[17].field_tag)
            _TIFFsetString(&xd->UserField18, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[18].field_tag)
            _TIFFsetString(&xd->UserField19, va_arg(ap, char*));
        else if (tag == xtiffFieldInfo[19].field_tag)
            _TIFFsetString(&xd->UserField20, va_arg(ap, char*));
        else
            return (PARENT(xt,vsetfield))(tif,tag,ap);

	/*
	switch (tag) {

                 // XXX put your extended tags here; replace the implemented
                 // example tags with your own.

	case xtiffFieldInfo[0].field_tag:
		_TIFFsetString(&xd->UserField1, va_arg(ap, char*));
		break;
	default:
		// call the inherited method
		return (PARENT(xt,vsetfield))(tif,tag,ap);
		break;
	}
        */

	if (status) {
		/* we have to override the print method here,
 		 * after the compression tags have gotten to it.
		 * This makes sense because the only time we would
		 * need the extended print method is if an extended
		 * tag is set by the reader.
		 */
		if (!(xt->xtif_flags & XTIFFP_PRINT))
		{
	        	PARENT(xt,printdir) =  TIFFMEMBER(tif,printdir);
      		  	TIFFMEMBER(tif,printdir) = _XTIFFPrintDirectory;
			xt->xtif_flags |= XTIFFP_PRINT;
		}
		TIFFSetFieldBit(tif, _TIFFFieldWithTag(tif, tag)->field_bit);
		tif->tif_flags |= TIFF_DIRTYDIRECT;
	}
	va_end(ap);
	return (status);
badvalue:
	TIFFError(tif->tif_name, "%d: Bad value for \"%s\"", v,
	    _TIFFFieldWithTag(tif, tag)->field_name);
	va_end(ap);
	return (0);
badvalue32:
	TIFFError(tif->tif_name, "%ld: Bad value for \"%s\"", v32,
	    _TIFFFieldWithTag(tif, tag)->field_name);
	va_end(ap);
	return (0);
}


static int
_XTIFFVGetField(TIFF* tif, ttag_t tag, va_list ap)
{
	xtiff *xt = XTIFFDIR(tif);
	XTIFFDirectory* xd = &xt->xtif_dir;

        // MB Jan 09, 2000 Use if statement to avoid having to use
        // constants in a case statement

        if (tag == xtiffFieldInfo[0].field_tag)
            *va_arg(ap, char**) = xd->UserField1;
        else if (tag == xtiffFieldInfo[1].field_tag)
            *va_arg(ap, char**) = xd->UserField2;
        else if (tag == xtiffFieldInfo[2].field_tag)
            *va_arg(ap, char**) = xd->UserField3;
        else if (tag == xtiffFieldInfo[3].field_tag)
            *va_arg(ap, char**) = xd->UserField4;
        else if (tag == xtiffFieldInfo[4].field_tag)
            *va_arg(ap, char**) = xd->UserField5;
        else if (tag == xtiffFieldInfo[5].field_tag)
            *va_arg(ap, char**) = xd->UserField6;
        else if (tag == xtiffFieldInfo[6].field_tag)
            *va_arg(ap, char**) = xd->UserField7;
        else if (tag == xtiffFieldInfo[7].field_tag)
            *va_arg(ap, char**) = xd->UserField8;
        else if (tag == xtiffFieldInfo[8].field_tag)
            *va_arg(ap, char**) = xd->UserField9;
        else if (tag == xtiffFieldInfo[9].field_tag)
            *va_arg(ap, char**) = xd->UserField10;
        else if (tag == xtiffFieldInfo[10].field_tag)
            *va_arg(ap, char**) = xd->UserField11;
        else if (tag == xtiffFieldInfo[11].field_tag)
            *va_arg(ap, char**) = xd->UserField12;
        else if (tag == xtiffFieldInfo[12].field_tag)
            *va_arg(ap, char**) = xd->UserField13;
        else if (tag == xtiffFieldInfo[13].field_tag)
            *va_arg(ap, char**) = xd->UserField14;
        else if (tag == xtiffFieldInfo[14].field_tag)
            *va_arg(ap, char**) = xd->UserField15;
        else if (tag == xtiffFieldInfo[15].field_tag)
            *va_arg(ap, char**) = xd->UserField16;
        else if (tag == xtiffFieldInfo[16].field_tag)
            *va_arg(ap, char**) = xd->UserField17;
        else if (tag == xtiffFieldInfo[17].field_tag)
            *va_arg(ap, char**) = xd->UserField18;
        else if (tag == xtiffFieldInfo[18].field_tag)
            *va_arg(ap, char**) = xd->UserField19;
        else if (tag == xtiffFieldInfo[19].field_tag)
            *va_arg(ap, char**) = xd->UserField20;
        else
            return (PARENT(xt,vgetfield))(tif,tag,ap);

        /*
	switch (tag) {

                 // XXX put your extended tags here; replace the implemented
                 // example tags with your own.

	case xtiffFieldInfo[0].field_tag:
		*va_arg(ap, char**) = xd->UserField1;
		break;
	default:
		// return inherited method
		return (PARENT(xt,vgetfield))(tif,tag,ap);
		break;
	}
        */
	return (1);
}

#define	CleanupField(member) {		\
    if (xd->member) {			\
	_TIFFfree(xd->member);		\
	xd->member = 0;			\
    }					\
}
/*
 * Release storage associated with a directory.
 */
static void
_XTIFFFreeDirectory(xtiff* xt)
{
	XTIFFDirectory* xd = &xt->xtif_dir;

	/* 
	 *  XXX - Purge all Your allocated memory except
	 *  for the xtiff directory itself. This includes
	 *  all fields that require a _TIFFsetXXX call in
	 *  _XTIFFVSetField().
	 */
	
	CleanupField(UserField1);
	
}
#undef CleanupField

static void _XTIFFLocalDefaultDirectory(TIFF *tif)
{
	xtiff *xt = XTIFFDIR(tif);
	XTIFFDirectory* xd = &xt->xtif_dir;

	/* Install the extended Tag field info */
	_TIFFMergeFieldInfo(tif, xtiffFieldInfo, N(xtiffFieldInfo));

	/*
	 *  free up any dynamically allocated arrays
	 *  before the new directory is read in.
	 */
	 
	_XTIFFFreeDirectory(xt);	
	_TIFFmemset(xt,0,sizeof(xtiff));

	/* Override the tag access methods */

	PARENT(xt,vsetfield) =  TIFFMEMBER(tif,vsetfield);
	TIFFMEMBER(tif,vsetfield) = _XTIFFVSetField;
	PARENT(xt,vgetfield) =  TIFFMEMBER(tif,vgetfield);
	TIFFMEMBER(tif,vgetfield) = _XTIFFVGetField;

	/* 
	 * XXX Set up any default values here.
	 */
	
	//xd->xd_example_single = 234;
}



/**********************************************************************
 *    Nothing below this line should need to be changed.
 **********************************************************************/

static TIFFExtendProc _ParentExtender;

/*
 *  This is the callback procedure, and is
 *  called by the DefaultDirectory method
 *  every time a new TIFF directory is opened.
 */

static void
_XTIFFDefaultDirectory(TIFF *tif)
{
	xtiff *xt;

	/* Allocate Directory Structure if first time, and install it */
	if (!(tif->tif_flags & XTIFF_INITIALIZED))
	{
		xt = _TIFFmalloc(sizeof(xtiff));
		if (!xt)
		{
			/* handle memory allocation failure here ! */
			return;
		}
		_TIFFmemset(xt,0,sizeof(xtiff));
		/*
		 * Install into TIFF structure.
		 */
		TIFFMEMBER(tif,clientdir) = (tidata_t)xt;
		tif->tif_flags |= XTIFF_INITIALIZED; /* dont do this again! */
	}
	
	/* set up our own defaults */
	_XTIFFLocalDefaultDirectory(tif);

	/* Since an XTIFF client module may have overridden
	 * the default directory method, we call it now to
	 * allow it to set up the rest of its own methods.
         */

	if (_ParentExtender)
		(*_ParentExtender)(tif);

}


/* MB Jan 09, 2000 Routine to set the tiff tag values */
void SetTiffTagValues( ttag_t UserTag1,
                       ttag_t UserTag2,
                       ttag_t UserTag3,
                       ttag_t UserTag4,
                       ttag_t UserTag5,
                       ttag_t UserTag6,
                       ttag_t UserTag7,
                       ttag_t UserTag8,
                       ttag_t UserTag9,
                       ttag_t UserTag10,
                       ttag_t UserTag11,
                       ttag_t UserTag12,
                       ttag_t UserTag13,
                       ttag_t UserTag14,
                       ttag_t UserTag15,
                       ttag_t UserTag16,
                       ttag_t UserTag17,
                       ttag_t UserTag18,
                       ttag_t UserTag19,
                       ttag_t UserTag20 )
{
    xtiffFieldInfo[0].field_tag = UserTag1;
    xtiffFieldInfo[1].field_tag = UserTag2;
    xtiffFieldInfo[2].field_tag = UserTag3;
    xtiffFieldInfo[3].field_tag = UserTag4;
    xtiffFieldInfo[4].field_tag = UserTag5;
    xtiffFieldInfo[5].field_tag = UserTag6;
    xtiffFieldInfo[6].field_tag = UserTag7;
    xtiffFieldInfo[7].field_tag = UserTag8;
    xtiffFieldInfo[8].field_tag = UserTag9;
    xtiffFieldInfo[9].field_tag = UserTag10;
    xtiffFieldInfo[10].field_tag = UserTag11;
    xtiffFieldInfo[11].field_tag = UserTag12;
    xtiffFieldInfo[12].field_tag = UserTag13;
    xtiffFieldInfo[13].field_tag = UserTag14;
    xtiffFieldInfo[14].field_tag = UserTag15;
    xtiffFieldInfo[15].field_tag = UserTag16;
    xtiffFieldInfo[16].field_tag = UserTag17;
    xtiffFieldInfo[17].field_tag = UserTag18;
    xtiffFieldInfo[18].field_tag = UserTag19;
    xtiffFieldInfo[19].field_tag = UserTag20;
}

/*
 *  XTIFF Initializer -- sets up the callback
 *   procedure for the TIFF module.
 */

static
void _XTIFFInitialize(void)
{
	static first_time=1;

	if (! first_time) return; /* Been there. Done that. */
	first_time = 0;

	/* Grab the inherited method and install */
	_ParentExtender = TIFFSetTagExtender(_XTIFFDefaultDirectory);
}


/*
 * Public File I/O Routines.
 */
TIFF*
XTIFFOpen(const char* name, const char* mode)
{
	/* Set up the callback */
	_XTIFFInitialize();

	/* Open the file; the callback will set everything up
	 */
	return TIFFOpen(name, mode);

}

TIFF*
XTIFFClientOpen(
	const char* name, const char* mode,
	thandle_t clientdata,
	TIFFReadWriteProc readproc,
	TIFFReadWriteProc writeproc,
	TIFFSeekProc seekproc,
	TIFFCloseProc closeproc,
	TIFFSizeProc sizeproc,
	TIFFMapFileProc mapproc,
	TIFFUnmapFileProc unmapproc
)
{
	/* Set up the callback */
	_XTIFFInitialize();

	/* Open the file; the callback will set everything up
	 */
	return TIFFClientOpen( name, mode, clientdata,
                               readproc, writeproc, seekproc,
                               closeproc, sizeproc,
                               mapproc, unmapproc);
}

void
XTIFFClose(TIFF *tif)
{
	xtiff *xt = XTIFFDIR(tif);

	/* call inherited function first */
	TIFFClose(tif);

	/* Free up extended allocated memory */
	_XTIFFFreeDirectory(xt);
	_TIFFfree(xt);
}

